@@ -66,6 +66,14 @@ class VersionedHDF5File:
         self._closed = False
         self._version_cache = {}
 
+    @property
+    def closed(self):
+        if self._closed:
+            return self._closed
+        if not self.f.id:
+            self._closed = True
+        return self._closed
+
     @property
     def current_version(self):
         """
@@ -83,21 +91,32 @@ class VersionedHDF5File:
         self._version_cache.clear()
 
     def get_version_by_name(self, version):
+        if version.startswith('/'):
+            raise ValueError("Versions cannot start with '/'. VersionedHDF5File should not be used to access the top-level of an h5py File.")
+
         if version == '':
             version = '__first_version__'
 
         if version not in self._versions:
             raise KeyError(f"Version {version!r} not found")
 
+        g = self._versions[version]
+        if not g.attrs['committed']:
+            raise ValueError("Version groups cannot accessed from the VersionedHDF5File object before they are committed.")
         # TODO: Don't give an in-memory group if the file is read-only
-        return InMemoryGroup(self._versions[version]._id, _committed=True)
+        return InMemoryGroup(g._id, _committed=True)
 
     def get_version_by_timestamp(self, timestamp, exact=False):
         version = get_version_by_timestamp(self.f, timestamp, exact=exact)
         # TODO: Don't give an in-memory group if the file is read-only
-        return InMemoryGroup(self._versions[version]._id, _committed=True)
+        g = self._versions[version]
+        if not g.attrs['committed']:
+            raise ValueError("Version groups cannot accessed from the VersionedHDF5File object before they are committed.")
+        return InMemoryGroup(g._id, _committed=True)
 
     def __getitem__(self, item):
+        if self.closed:
+            raise ValueError("File is closed")
         if item in self._version_cache:
             # We don't cache version names because those are already cheap to
             # lookup.
@@ -160,7 +179,7 @@ class VersionedHDF5File:
         `prev_version` for any future `stage_version` call.
 
         """
-        if self._closed:
+        if self.closed:
             raise ValueError("File is closed")
         old_current = self.current_version
         group = create_version_group(self.f, version_name,
@@ -196,7 +215,7 @@ class VersionedHDF5File:
 
         These messages are intended to be similar to h5py messages.
         """
-        if self._closed:
+        if self.closed:
             return "<Closed VersionedHDF5File>"
         else:
             return f"<VersionedHDF5File object \"{self.f.filename}\" (mode" \
